function [d,id]=qpdes(Z,LT,g,a,w,ipvt,n,ng,ind,info,id)
%%%%%%%%%%%%
%Author:Carlos Crespo under the direction of Cecilia Pola
%Date: 28/1/2021
%Reference: desr03 FORTRAN function of Eduardo Casas and Cecilia Pola.
%
% Purpose : this program calculates a down direction for a quadratic
% problem
%
%Input List:  -LT: An NxN upper triangular matrix which is calculated in
%                  FactCholPiv.
%             -Z: An NGxN matrix whose columns form a basis of the kernel
%             of the matrix.
%             -a: A vector of dimension ng containing the coefficients of
%                 the last constraint removed from the active set if Id<-NG or
%                 Id>Ng.
%             -w: A work vector of dimension 2*N. If info=10 in the first n
%             coordinates it has the projected gradient
%             -ipvt: A vector of dimension N containing ipvt vector
%                   of FactCholPiv
%             -n: Number of columns of the matrix Z
%             -ng: Dimension of gradient vector
%             -ind: A number that indicates which type of factorization has been produced
%                  in FactCholPiv
%             -info: A variable that can take different values
%                        0: The projected gradient is any vector
%                        1: The projected gradient is a scalar multiple
%                           of the n-th vector of the canonical basis
%                       10: The projected gradient is given
%             -id: A variable that can take different values
%                    <> 0:
%                       0: In other case
%
%Output List: -d: Vector containing the down direction
%             -id: Variable which indicates how the down direction has been
%             calculated
%                  0: negative or null curvature direction
%                  1: other case
%             -ro
%
%%%%%%%%%%%%
Z=Z(:,n:-1:1);
d=zeros(ng,1);
eps1=eps^.75;
indmul=id;
id=0;
s=0;%
%g=zeros(ng,1);%
%In the case of ind>=0 the projected gradient is calculated (positive
% semidefinite case)
if ind>=0
    if info==0
        w(1:n)=-Z(:,1:n)'*g;
    elseif info==1
        wn=-Z(:,n)'*g;
    elseif info==10
        info=0;
        w(1:n)=-w(1:n);
    end
end
% Projected Hessian positive definite
if ind==0
    id=1;
    if info==0
        [w(1:n),ifallo]=ansr02(LT(1:n,1:n),w(1:n),ipvt);
        if ifallo==1
            return;
        end
    else
        y=zeros(n,1);
        y(n)=wn/LT(n,n);%-
        [w(1:n),ifallo]=solupptri(LT(1:n,1:n),y);
        if ifallo==1
            return;
        end
    end
    % Projected Hessian non-definite. In this case a direction of negative
    % curvature is calculated
elseif ind< -1 && ind>=-n
    m=-ind;
    m2=m-1;
    B=-LT(1:m2,m);
    [d(1:m2),ifail]=solupptri(LT(1:m2,1:m2),B);
    if ifail==0
        w(1:n)=0;
        for i=1:m2
            w(ipvt(i))=d(i);
        end
        w(ipvt(m))=1;
    else
        fprint('Error');
        return;
    end
elseif ind<-n
    m=-ind-n;
    m1=m+1;
    Hrs=LT(m,m1);
    B=-LT(1:m-1,m1)+Hrs*LT(1:m-1,m);
    if m>1
        [d(1:m-1),ifail]=solupptri(LT(1:m-1,1:m-1),B);
    end
    d(m)=-Hrs;
    for i=1:m
        w(ipvt(i))=d(i);
    end
    w(ipvt(m1))=1;
    % Hessian semi-definite positive case
elseif ind>0 && ind<n
    k=0;
    m=n-ind;
    if info==0
        w(n+1:2*n)=w(ipvt(1:n));
    else
        i=1;
        while n~=ipvt(i)
            i=i+1;
        end
    end
    % A base of the projected Hessian kernel is computed
    for j=1:ind
        mj=m+j;
        [w(1:m),ifail]=solupptri(LT(1:m,1:m),LT(1:m,mj));
        'base'
        ipvt
        w(1:n)
        if ifail==1
            return;
        end
        if info==0
            s=w(1:m)'*w(n+1:n+m)-w(mj+n);
        else
            if i==mj
                s=-wn;
            elseif i<=m
                s=w(i)*wn;
            else
                s=0;
            end
        end
        if abs(s)>eps1
            k=1;
            d(1:m)=d(1:m)+s*w(1:m);
            d(mj)=-s;
            for i=1:n
                w(ipvt(i))=d(i);
            end
        end
    end
    k=k;
    %If k=0 means that the projected gradient is orthogonal to the base of
    %the kernel. A down direction is computed in this case
    if k==0
        id=1;
        if info==0
            for i=1:n
                w(ipvt(i))=w(n+i);
            end
            [w(1:n),ifallo]=ansr02(LT(1:n,1:n),w(1:n),ipvt(1:m));%%
            w(ipvt(m+1:n))=0;
        else
            d(m)=-1;
            d(1:m-1)=0;
            [d(1:m),ifail]=solupptri(LT(1:m,1:m),d(1:m));
            d(m+1:n)=0;
            for i=1:n
                w(ipvt(i))=d(i);
            end
        end
    end
end
% The obtained direction is projected
if ind==n && info==1
    if abs(wn)>eps1
        d(1:ng)=wn*Z(1:ng,1);
    else
        id=1;
        d(1:ng)=0;
    end
elseif ind==-1
    d(1:ng)=Z(1:ng,ipvt(1));
else
    if ind==n
        s=norm(w(1:n));
        if s<=eps1
            id=1;
            d(1:ng)=0;
        end
    end
    if ind~=n || (ind==n && id==0)
        d(1:ng)=Z(1:ng,:)*w(1:n);
    end
end
% The direction calculated is affected with the appropriate sign to
% make it a down direction
if ind<0 || (id==1 && info ==1 && ind>0)
    if id==1
        s=d'*g;
        if s>0
            d(1:ng)=-d(1:ng);
        else
            s=-s;
        end
    else
        if indmul==0
            s=d'*g;
        else
            if indmul >ng || indmul <-ng
                s=d'*a;
            else
                s=d(abs(indmul));
            end
        end
        if (indmul>=0 && s>0)||(indmul<0 && s<0)
            d(1:ng)=-d(1:ng);
        end
    end
end
   
end

